PROGRAM ColorQuixes;

(* This program will draw several sets of colored lines (quixes) which     *)
(* bound around the screen in parrallel.  The program runs until the mouse *)
(* button or any key on the keyboard is pressed.                           *)
(*                                                                         *)
(* The following program constants can be altered for interesting results: *)
(*                                                                         *)
(*    QuixCount    -- Controls the number of quixes                        *)
(*    MaxQuixDepth -- Number of lines wide for each quix                   *)
(*
(* This program was written under Turbo Pascal 1.1 and will run on should  *)
(* work on any Macintosh.  If color is available, the each set of lines    *)
(* will be a different color.                                              *)
(*                                                                         *)
(* This program was written by Chris Zagar (Compuserve 74216,2456) and was *)
(* last modified on March 12, 1988.                                        *)

{$U-}

USES
  Memtypes, QuickDraw, OSIntf, ToolIntf;

CONST
  QuixCount = 4;
  MaxColors = 7;
  MaxQuixDepth = 7;

VAR
  gPort:   GrafPort;
  NextEvent: EventRecord;
  MaxX, MaxY: INTEGER;
  X1, Y1, X2, Y2: ARRAY [1..QuixCount, 1..MaxQuixDepth] OF INTEGER;
  iX1, iY1, iX2, iY2: ARRAY [1..QuixCount] OF INTEGER;   
  Loop, Last: INTEGER;
  Temp: INTEGER;
  ColorTable: ARRAY [1..7] OF INTEGER;
      
PROCEDURE Initialize;  

BEGIN (* Initialiaze *)
  InitGraf(@thePort);
  InitCursor;
  HideCursor;
  OpenPort(@gPort);
  PenPat(White);
  BackPat(Black);
  EraseRect(gPort.portRect);
  FrameRect(gPort.portRect);
  WITH gPort.portBits.Bounds DO
    BEGIN
      MaxX := Right - Left - 2;
      MaxY := Bottom - Top - 2
    END; (* WITH *)
  FOR Temp := 1 TO QuixCount DO
    BEGIN
      X1[Temp, 1] := 2 + ((Temp * 20) MOD MaxX);
      Y1[Temp, 1] := 2;
      X2[Temp, 1] := 2 + (((MaxX DIV 2) + Temp * 20) MOD MaxX);
      Y2[Temp, 1] := (MaxY DIV 2);
      iX1[Temp] := 2;
      iX2[Temp] := 3;
      iY1[Temp] := 5;
      iY2[Temp] := 7; 
      FOR Loop := 2 TO MaxColors DO
        BEGIN
          X1[Temp, Loop] := 0;
          X2[Temp, Loop] := 0;
          Y1[Temp, Loop] := 0;
          Y2[Temp, Loop] := 0
        END (* FOR *)
    END; (* FOR *)
  Loop := 1;
  Last := 2;
  ColorTable[1] := CyanColor;
  ColorTable[2] := RedColor;
  ColorTable[3] := GreenColor;
  ColorTable[4] := BlueColor;
  ColorTable[5] := WhiteColor;
  ColorTable[6] := MagentaColor;
  ColorTable[7] := YellowColor
END; (* Initialize *)

PROCEDURE Inc(VAR ValNew: INTEGER;
                  ValOld: INTEGER;
              VAR ValInc: INTEGER;
                  ValLow: INTEGER;
                  ValHigh: INTEGER);
             
VAR
  ValTemp: INTEGER;
              
BEGIN (* Inc *)
  ValTemp := ValOld + ValInc;
  IF (ValTemp >= ValLow) AND (ValTemp <= ValHigh)
    THEN ValNew := ValTemp
    ELSE
      BEGIN
        ValInc := - ValInc;
        ValNew := ValOld + ValInc
      END (* ELSE *)
END; (* Inc *)

BEGIN (* Quix 7 *)
  Initialize;
  WHILE NOT OSEventAvail(mDownMask + keyDownMask, NextEvent) DO
    BEGIN
      FOR Temp := 1 TO QuixCount DO
        BEGIN
          BackColor(ColorTable[(Temp MOD MaxColors) + 1]);
          MoveTo(X1[Temp, Loop], Y1[Temp, Loop]);
          LineTo(X2[Temp, Loop], Y2[Temp, Loop]);
        END; (* FOR *)
      Last := Loop;
      IF Loop = MaxQuixDepth
        THEN Loop := 1
        ELSE Loop := Loop + 1;
      BackColor(BlackColor);
      FOR Temp := 1 TO QuixCount DO
        BEGIN
         MoveTo(X1[Temp, Loop], Y1[Temp, Loop]);
         LineTo(X2[Temp, Loop], Y2[Temp, Loop]);
         Inc(X1[Temp, Loop], X1[Temp, Last], iX1[Temp], 2, MaxX);
         Inc(X2[Temp, Loop], X2[Temp, Last], iX2[Temp], 2, MaxX);
         Inc(Y1[Temp, Loop], Y1[Temp, Last], iY1[Temp], 2, MaxY);
         Inc(Y2[Temp, Loop], Y2[Temp, Last], iY2[Temp], 2, MaxY)
      END (* FOR *)
    END (* WHILE *)
END. (* ColorQuixes *)

